﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class select : MonoBehaviour {

	//a list of all selectable objects. may have to make it to scan for all selectable objects by tag or name
	//rather then to manually select them. I mean, there may be troops spawining.
	public List<Transform>  selectables;

	//their current positions in screenspace
	public List<Vector3> positionsOnScreen;

	//the selected objects
	public List<Transform>  selected = new List<Transform>();

	//selection
	//the selection rectangle
	Rect selectionRect;
	//first screen position where left mouse button was clicked
	Vector2 originalPos;
	//second screen position where left mouse button was released
	Vector2 finalPos;
	//the reactangle that will be displayed on the screen
	Rect displayRect;
	//tells gui whether to draw the selection rectangle
	bool drawRect = false;

	//these are only for selection with click ability
	RaycastHit hit;
	Ray ray;
	float rayDistance = 100f;

	// Use this for initialization
	void Start () {

	}
	
	// Update is called once per frame
	void Update () {
		//fromWorldToScreen();
		//selection
		if(Input.GetMouseButtonDown(0))
		{
			//if left mouse button pressed down, record the cursor position
			//for the starting point of selection rectangle
			originalPos.x = Input.mousePosition.x;
			originalPos.y = Input.mousePosition.y;
		}
		else if(Input.GetMouseButton(0))
		{
			//while left mouse button down, record the final cursor position.
			//maybe a more efficient way is to make it with event.MouseonMove or similar,though it works this way just fine
			finalPos.x = Input.mousePosition.x;
			finalPos.y = Input.mousePosition.y;

			//calculate the rectangle made up from the original and final mouse positions
			selectionRect = new Rect(Mathf.Min(originalPos.x, finalPos.x), Mathf.Min(originalPos.y, finalPos.y), Mathf.Abs(originalPos.x - finalPos.x), Mathf.Abs(originalPos.y - finalPos.y));
			//the reactangle that will be displayed on the screen
			displayRect = new Rect(Mathf.Min(originalPos.x, finalPos.x), Mathf.Min(Screen.height - originalPos.y, Screen.height - finalPos.y), Mathf.Abs(originalPos.x - finalPos.x), Mathf.Abs((Screen.height - originalPos.y) - (Screen.height - finalPos.y)));

			//tell gui to start drawing the rectangle
			drawRect = true;
		}
		else if(Input.GetMouseButtonUp(0))
		{

			//when left mouse button released, calculate which selectables are grasped and add them to selected[],
			//but not before selected is cleared from the previous selection, unless ctrl is pressed before left mouse button
			//is released, in which case the new selection is added to the previous selection

			//clears the previous selection
			//this first if() is for the first time you select anything, so that it doesnt give out exceptions and stuff
			if(selected.Count>0)
			{
				//if ctrl is pressed, dont clear the previous selection,
				//but instead add this new selection to the old one
				if(!Input.GetKey("left ctrl"))
				{
					foreach(Transform objecT in selected)
					{
						objecT.renderer.material.color = Color.blue;
					}
					selected.Clear();
				}
			}
			//check which selectables are grasped with the selection rectangle (according to their pivots)
			foreach(Transform objecT in selectables)
			{
				//if objects are within, add the selected objects to the selected[] list
				if(selectionRect.Contains(camera.WorldToScreenPoint(objecT.position)))
				{
					selected.Add(objecT);
					//change its color so that it is obvious it was selected
					selected.Find(delegate(Transform bk){return bk == objecT;}).renderer.material.color = Color.green;
				}
			}
			//this code adds functionality of selection with just a click, without rectangle:
			ray = Camera.main.ScreenPointToRay(Input.mousePosition);
			if(Physics.Raycast (ray, out hit, rayDistance))
			{
				if(hit.transform.tag == "troop")
				{
					//if ctrl is pressed, dont clear the previous selection,
					//but instead add this new selection to the old one
					if(!Input.GetKey("left ctrl"))
					{
						foreach(Transform objecT in selected)
						{
							objecT.renderer.material.color = Color.blue;
						}
						selected.Clear();
					}
					//add clicked object
					selected.Add(hit.transform);
					//change its color so that it is obvious it was selected
					selected.Find(delegate(Transform bk){return bk == hit.transform;}).renderer.material.color = Color.green;
				}
			}


			//tells the gui to stop drawing the rectangle
			drawRect = false;
		}
	}

	//display of the selection rectangle. for some reason it behaves y-reverse  from the expected and from the actual selection.
	public Texture2D texture;
	void DrawQuad(Rect position, Color color) {
		//Texture2D texture = new Texture2D(1, 1);
		//texture.SetPixel(0,0,color);
		texture.Apply();
		GUI.skin.box.normal.background = texture;
		GUI.skin.box.border = new RectOffset(6,6,6,6);
	
		GUI.Box(position, GUIContent.none);
	}

	void OnGUI () {
		//while mouse button is held down, draw the selection rectangle
		if(drawRect)
		{
			DrawQuad(displayRect, Color.red);
		}

	}

}
